/////////////////////////////////////////////////////////////////////////////////////
//
//  Code written to run off a CFA-10006 which uses an ATMEGA2561 running at 8 MHz.
//
/////////////////////////////////////////////////////////////////////////////////////
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>

// control lines
#define	LCD_RESET	PC2
#define	LCD_RD    PC5
#define	LCD_WR    PC6
#define	LCD_RS	  PC7   
#define	LCD_CS	  PC4
#define LED_1     PD1
#define LED_2     PD2

// macros for control lines
#define CLR_RS PORTC &= ~(1<<LCD_RS);
#define SET_RS PORTC |=  (1<<LCD_RS);

#define CLR_CS PORTC &= ~(1<<LCD_CS);
#define SET_CS PORTC |=  (1<<LCD_CS);

#define CLR_RESET PORTC &= ~(1<<LCD_RESET);
#define SET_RESET PORTC |=  (1<<LCD_RESET);

#define CLR_RW PORTC &= ~(1 << LCD_WR);
#define SET_RW PORTC |=  (1 << LCD_WR);
#define CLR_E PORTC &= ~(1 << LCD_RD);
#define SET_E PORTC |=  (1 << LCD_RD);

#define LED1_OFF PORTD &= ~(1 << LED_1);
#define LED1_ON  PORTD |=  (1 << LED_1);
#define LED2_OFF PORTD &= ~(1 << LED_2);
#define LED2_ON  PORTD |=  (1 << LED_2);

#define uint8 unsigned char
#define uint16 unsigned int
#define uint32 unsigned long


// ************************************************************************* //
void checkbusy()
{
	DDRA=0x00;
	PORTA=0x00;
	CLR_RS;
	SET_RW;
	do
	{
		SET_E;
		_delay_us(100);
		CLR_E;
	}while(PA7==1);

	DDRA=0xFF;
}

// ************************************************************************* //
void write_command(unsigned char command)
{
	checkbusy();

  PORTA=command;// write the command to the port

  CLR_CS;       // chip selected
  CLR_RS;       // set to write to the control registers
	CLR_RW;

	_delay_us(100);
	SET_E;       // clock in the command
	_delay_us(100);
	CLR_E;       // bring write line low

  SET_CS;       // unselect chip
}
// ************************************************************************* //
void write_data(unsigned char data)
{

	checkbusy();

  PORTA=data;// write the command to the port

  CLR_CS;       // chip selected
  SET_RS;       // set to write to the control registers
	CLR_RW;

	_delay_us(100);
  SET_E;       // clock in the command
	_delay_us(100);
		CLR_E;       // bring write line low

  SET_CS;       // unselect chip
}


// ************************************************************************* //
void CP_COMMAND(unsigned char comm_reg, unsigned char data_reg)
  {
  write_command(comm_reg);
  write_data(data_reg);
  }
// ************************************************************************* //
void initialize_display()
{
	CLR_E;
	CLR_RS;
	SET_CS;
	PORTA=0x00;

	SET_RESET;
	_delay_ms(10);
	CLR_RESET;
	_delay_ms(100);
	SET_RESET;
	_delay_ms(500);


	// function set
	// 0x38 == 8 bit mode, 5x8 character, japanese and english cgrom
	// 0x39 == 8 bit mode, 5x8 character, western european cgrom
	// 0x3A == 8 bit mode, 5x8 character, english and russion (cyrillic) cgrom
	write_command(0x38);
	_delay_us(100);

	// 0x0C Display on, cursor off, not blinking
	// 0x0E display on, cursor on, not blinking (underscore)
	// 0x0F display on, cursor on, blinking
	write_command(0x0C);
	_delay_us(100);

	// 0x06 is Entry mode set, increment, no shift
	write_command(0x06);
	_delay_us(100);

	// cursor home
	write_command(0x02);
	_delay_us(100);

	// clear display
	write_command(0x01);
	_delay_us(100);

	//				      01234567890123456789
	char line1[21]="CRYSTALFONTZ";
	char line2[21]="*CFAL1202A**";


	// write to the first line
	write_command(0x80);
	for(int col=0;col<20;col++)
	{
		write_data(line1[col]);
	}

	// write to the first line
	write_command(0xC0);
	for(int col=0;col<20;col++)
	{
		write_data(line2[col]);
	}

	write_command(0x02);
	_delay_us(100);

}

int main()
{
  DDRC=0xFF;    // all pins as outputs
  DDRA=0xFF;    // all pins as outputs
	DDRD |= 0x06;	// Switches as inputs, LEDs as outputs

  PORTA=0xFF;   // data lines low
  PORTC=0xFF;   // control and other lines high
  PORTD=0xF0;   // pull ups on the keys, remaining pins low

	// Initialize the display
	initialize_display();

	while (1)
	{
		;
	}
	return 0;
}

